MODULE merge_module
!
!  This module contains a merge subroutine to
!  provide an explicit interface.
!
IMPLICIT NONE

! Now add methods
CONTAINS

SUBROUTINE merge(b1, size1, b2, size2, out, size_out)
! 
! Subroutine to merge two sorted arrays together
! in increasing order.
!
!  Record of revisions:
!      Date       Programmer          Description of change
!      ====       ==========          =====================
!    05/06/16    S. J. Chapman        Original code
!
IMPLICIT NONE

! Declare calling arguments
INTEGER :: size1                  ! Size of array b1
REAL,DIMENSION(size1) :: b1       ! Input array b1
INTEGER :: size2                  ! Size of array b2
REAL,DIMENSION(size1) :: b2       ! Input array b1
INTEGER :: size_out               ! Size of array out
REAL,DIMENSION(size_out) :: out   ! Output array b1

! Declare local variables
INTEGER :: i1                     ! Pointer in b1
INTEGER :: i2                     ! Pointer in b2
INTEGER :: iout                   ! Pointer in out

! Initialise pointers
i1   = 1
i2   = 1
iout = 1

! Now do the merge, putting the smaller value
! from either input into the output array at
! each step.
DO
   IF ( iout > size_out ) THEN
   
      ! All done, get out.
      EXIT
      
   ELSE IF ( i1 > size1 ) THEN
   
      ! If b1 is finished, use b2
      out(iout) = b2(i2)
      iout = iout + 1
      i2   = i2 + 1
      
   ELSE IF ( i2 > size2 ) THEN

      ! If b2 is finished, use b1
      out(iout) = b2(i1)
      iout = iout + 1
      i1   = i1 + 1
      
   ELSE IF ( b1(i1) <= b2(i2) ) THEN

      ! If b1 is smaller, use it
      out(iout) = b1(i1)
      iout = iout + 1
      i1   = i1 + 1
      
   ELSE IF ( b1(i1) > b2(i2) ) THEN
   
      ! If b2 is smaller, use it
      out(iout) = b2(i2)
      iout = iout + 1
      i2   = i2 + 1
      
   END IF
END DO

END SUBROUTINE merge
   
END MODULE merge_module
